package models

import (
	"eta/eta_index_lib/utils"
	"fmt"
	"strconv"
	"strings"
	"time"

	"github.com/beego/beego/v2/client/orm"
)

// BaseFromGprRisk 美国农业部
type BaseFromGprRisk struct{}

type BaseFromGprRiskData struct {
	BaseFromGprRiskDataId  int `orm:"column(base_from_gpr_risk_data_id);pk"`
	BaseFromGprRiskIndexId int
	IndexCode              string
	DataTime               string
	Value                  string
	CreateTime             time.Time
	ModifyTime             time.Time
	DataTimestamp          int64
}

func GetBaseFromGprRiskDataByCondition(condition string, pars []interface{}) (list []*BaseFromGprRiskData, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM base_from_gpr_risk_data WHERE 1=1 `
	if condition != "" {
		sql += condition
	}
	_, err = o.Raw(sql, pars).QueryRows(&list)
	return
}

// Add 添加
func (obj BaseFromGprRisk) Add(edbCode string) (err error) {
	o := orm.NewOrm()

	var condition string
	var pars []interface{}
	if edbCode != "" {
		condition += " AND index_code=? "
		pars = append(pars, edbCode)
	}
	GprRiskBaseDataAll, err := GetBaseFromGprRiskDataByCondition(condition, pars)
	if err != nil && err.Error() != utils.ErrNoRow() {
		return
	}
	var isAdd bool
	addSql := ` INSERT INTO edb_data_gpr_risk(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	existMap := make(map[string]string)
	for _, sv := range GprRiskBaseDataAll {
		eDate := sv.DataTime
		dataTime, err := time.Parse(utils.FormatDate, eDate)
		if err != nil {
			return err
		}
		timestamp := dataTime.UnixNano() / 1e6
		timeStr := fmt.Sprintf("%d", timestamp)
		if _, ok := existMap[eDate]; !ok {
			addSql += GetAddSql("0", edbCode, eDate, timeStr, sv.Value)
			isAdd = true
		}
		existMap[eDate] = sv.Value
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		utils.FileLog.Info("addSql:" + addSql)
		_, err = o.Raw(addSql).Exec()
		if err != nil {
			return err
		}
	}
	return
}

// Refresh 刷新涌益咨询指标数据
func (obj BaseFromGprRisk) Refresh(edbInfoId int, edbCode, startDate string) (err error) {
	source := obj.GetSource()
	o := orm.NewOrm()
	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据
	var condition string
	var pars []interface{}

	if edbCode != "" {
		condition += " AND index_code=? "
		pars = append(pars, edbCode)
	}

	if startDate != "" {
		condition += " AND data_time>=? "
		pars = append(pars, startDate)
	}

	GprRiskDataList, err := GetBaseFromGprRiskDataByCondition(condition, pars)
	if err != nil {
		return
	}

	// 真实数据的最大日期  ， 插入规则配置的日期
	var realDataMaxDate, edbDataInsertConfigDate time.Time
	var edbDataInsertConfig *EdbDataInsertConfig
	var isFindConfigDateRealData bool //是否找到配置日期的实际数据的值
	{
		edbDataInsertConfig, err = GetEdbDataInsertConfigByEdbId(edbInfoId)
		if err != nil && err.Error() != utils.ErrNoRow() {
			return
		}
		if edbDataInsertConfig != nil {
			edbDataInsertConfigDate = edbDataInsertConfig.Date
		}
	}

	var existCondition string
	var existPars []interface{}

	existCondition += " AND edb_info_id=? "
	existPars = append(existPars, edbInfoId)
	if startDate != "" {
		existCondition += " AND data_time>=? "
		existPars = append(existPars, startDate)
	}

	existList, err := GetEdbDataByCondition(source, 0, existCondition, existPars)
	if err != nil {
		return err
	}
	existMap := make(map[string]*EdbInfoSearchData)
	for _, v := range existList {
		existMap[v.DataTime] = v
	}
	addSql := ` INSERT INTO edb_data_gpr_risk(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	var isAdd bool
	for _, v := range GprRiskDataList {
		item := v
		eDate := item.DataTime
		dataTime, err := time.ParseInLocation(utils.FormatDate, eDate, time.Local)
		if err != nil {
			return err
		}
		if findItem, ok := existMap[v.DataTime]; !ok {
			sValue := item.Value

			timestamp := dataTime.UnixNano() / 1e6
			timeStr := fmt.Sprintf("%d", timestamp)

			addSql += GetAddSql(edbInfoIdStr, edbCode, eDate, timeStr, sValue)
			isAdd = true
		} else {
			if findItem != nil && utils.SubFloatToString(findItem.Value, 30) != item.Value {
				err = ModifyEdbDataById(source, 0, findItem.EdbDataId, item.Value)
				if err != nil {
					return err
				}
			}
		}

		// 下面代码主要目的是处理掉手动插入的数据判断
		{
			if realDataMaxDate.IsZero() || dataTime.After(realDataMaxDate) {
				realDataMaxDate = dataTime
			}
			if edbDataInsertConfigDate.IsZero() || dataTime.Equal(edbDataInsertConfigDate) {
				isFindConfigDateRealData = true
			}
		}
	}

	// 处理手工数据补充的配置
	HandleConfigInsertEdbData(realDataMaxDate, edbDataInsertConfig, edbInfoId, source, 0, existMap, isFindConfigDateRealData)

	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = o.Raw(addSql).Exec()
		if err != nil {
			fmt.Println("RefreshEdbDataFromBaiinfo add Err", err.Error())
			return
		}
	}
	return
}

// GetSource 获取来源编码id
func (obj BaseFromGprRisk) GetSource() int {
	return utils.DATA_SOURCE_GPR_RISK
}

// GetSourceName 获取来源名称
func (obj BaseFromGprRisk) GetSourceName() string {
	return utils.DATA_SOURCE_NAME_GPR_RISK
}

type BaseFromGprRiskIndex struct {
	BaseFromGprRiskIndexId int64 `orm:"column(base_from_gpr_risk_index_id);pk"`
	IndexCode              string
	IndexName              string
	Frequency              string
	Unit                   string
	StartDate              string
	EndDate                string
	ClassifyId             int64
	Sort                   int
	BaseFileName           string
	RenameFileName         string
	TerminalCode           string
	CreateTime             time.Time
	ModifyTime             time.Time
}

type BaseFromGprRiskIndexList struct {
	BaseFromGprRiskIndexId int64 `orm:"column(base_from_gpr_risk_index_id);pk"`
	IndexCode              string
	IndexName              string
	Frequency              string
	Unit                   string
	Sort                   int
	ClassifyId             int64
	StartDate              string
	EndDate                string
	TerminalCode           string
	CreateTime             string
	ModifyTime             string
}

func (y *BaseFromGprRiskData) GetByIndexCode(indexCode string) (list []*BaseFromGprRiskData, err error) {
	o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_gpr_risk_data WHERE index_code=? `
	_, err = o.Raw(sql, indexCode).QueryRows(&list)
	return
}

func (y *BaseFromGprRiskData) AddMulti(item []*BaseFromGprRiskData) (err error) {
	o := orm.NewOrm()
	_, err = o.InsertMulti(1, item)
	return
}

// Update 修改
func (y *BaseFromGprRiskData) Update(updateCols []string) (err error) {
	o := orm.NewOrm()
	_, err = o.Update(y, updateCols...)
	return
}

// HandleGprRiskExcelData 涌益咨询的excel数据
type HandleGprRiskExcelData struct {
	ClassifyName       string `description:"指标目录"`
	ParentClassifyName string `description:"父级指标目录"`
	ClassifySort       int    `description:"指标目录排序号"`
	IndexName          string `description:"指标名称"`
	IndexCode          string `description:"指标编码"`
	Unit               string `description:"单位"`
	Sort               int    `description:"排序号"`
	Frequency          string `description:"频度"`
	ExcelDataMap       map[string]string
}

type HandleGprRiskExcelDataReq struct {
	List         []*HandleGprRiskExcelData
	TerminalCode string `description:"编码"`
}

func (y *BaseFromGprRiskData) GetMaxAndMinDateByIndexCode(indexCode string) (item *EdbInfoMaxAndMinInfo, err error) {
	o := orm.NewOrm()
	sql := ` SELECT MIN(data_time) AS min_date,MAX(data_time) AS max_date,MIN(value) AS min_value,MAX(value) AS max_value FROM base_from_gpr_risk_data WHERE index_code=? `
	err = o.Raw(sql, indexCode).QueryRow(&item)
	var latest_value float64
	sql = ` SELECT value AS latest_value FROM %s WHERE index_code=? ORDER BY data_time DESC LIMIT 1 `
	sql = fmt.Sprintf(sql, "base_from_gpr_risk_data")
	err = o.Raw(sql, indexCode).QueryRow(&latest_value)
	item.LatestValue = latest_value
	return
}

func (y *BaseFromGprRiskIndex) ModifyIndexMaxAndMinDate(indexCode string, item *EdbInfoMaxAndMinInfo) (err error) {
	o := orm.NewOrm()
	sql := ` UPDATE base_from_gpr_risk_index SET start_date=?,end_date=?, end_value=?, modify_time=NOW() WHERE index_code=? `
	_, err = o.Raw(sql, item.MinDate, item.MaxDate, item.LatestValue, indexCode).Exec()
	return
}

func (y *BaseFromGprRiskIndex) GetByIndexCode(indexCode string) (item *BaseFromGprRiskIndex, err error) {
	o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_gpr_risk_index WHERE index_code=? `
	err = o.Raw(sql, indexCode).QueryRow(&item)
	return
}

func (y *BaseFromGprRiskIndex) Add() (lastId int64, err error) {
	o := orm.NewOrm()
	lastId, err = o.Insert(y)
	return
}

// Update 修改
func (y *BaseFromGprRiskIndex) Update(updateCols []string) (err error) {
	o := orm.NewOrm()
	_, err = o.Update(y, updateCols...)

	return
}

// BaseFromGprRiskClassify GprRisk原始数据分类表
type BaseFromGprRiskClassify struct {
	ClassifyId      int64     `orm:"column(classify_id);pk"`
	ClassifyName    string    `description:"分类名称"`
	ClassifyNameEn  string    `description:"分类名称"`
	ParentId        int       `description:"父级id"`
	SysUserId       int       `description:"创建人id"`
	SysUserRealName string    `description:"创建人姓名"`
	Level           int       `description:"层级"`
	Sort            int       `description:"排序字段，越小越靠前，默认值：10"`
	ModifyTime      time.Time `description:"修改时间"`
	CreateTime      time.Time `description:"创建时间"`
}

func (y *BaseFromGprRiskClassify) Add() (lastId int64, err error) {
	o := orm.NewOrm()
	lastId, err = o.Insert(y)
	return
}

// Update 修改
func (y *BaseFromGprRiskClassify) Update(updateCols []string) (err error) {
	o := orm.NewOrm()
	_, err = o.Update(y, updateCols...)

	return
}

func (y *BaseFromGprRiskClassify) GetByClassifyName(classifyName string) (item *BaseFromGprRiskClassify, err error) {
	o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_gpr_risk_classify WHERE classify_name=? `
	err = o.Raw(sql, classifyName).QueryRow(&item)
	return
}

func (y *BaseFromGprRiskClassify) GetParentClassify() (items []*BaseFromGprRiskClassify, err error) {
	o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_gpr_risk_classify WHERE parent_id=0 `
	_, err = o.Raw(sql).QueryRows(&items)
	return
}
